﻿using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;

namespace OBSBlur.Window;

public class WindowManager : IDisposable
{
    public readonly HashSet<WindowInfo> Windows = new();
    public int UpdateInterval = 10;
    private bool KeepUpdating = true;
    public WindowManager()
    {
        Thread t = new (() =>
        {
            while(KeepUpdating)
                EnumWindows(WindowPlacement, 0);
            Thread.Sleep(UpdateInterval);
        });
        t.Start();
    }

    private bool WindowPlacement(IntPtr windowHandle, IntPtr lParam)
    {
        WINDOWPLACEMENT windowPlacement = new ();
        GetWindowPlacement(windowHandle, ref windowPlacement);
        Rectangle n = windowPlacement.NormalPosition;
        //Do not add if Window is not a drawable
        if (n is { Left: 0, Top: 0, Right: 0, Bottom: 0 })
            return true;
        
        uint pid;
        GetWindowThreadProcessId(windowHandle, out pid);
        Process processInfo = Process.GetProcessById((int)pid);
        //nvcontainer does not have a window name
        if (processInfo.ProcessName.Equals("nvcontainer"))
            return true;
        string windowTitle = GetWindowTextRaw(windowHandle);
        //Do not add, if window does not have a name
        if (windowTitle.Length < 1)
            return true;

        Windows.Add(new WindowInfo(windowHandle, windowTitle, processInfo, windowPlacement));
        return true;
    }
    
    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    static extern bool EnumWindows(EnumWindowsProc lpEnumFunc, IntPtr lParam);

    private delegate bool EnumWindowsProc(IntPtr windowHandle, IntPtr lParam);
    
    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    public static extern bool GetWindowPlacement(IntPtr windowHandle, ref WINDOWPLACEMENT lpwndpl);
    
    [DllImport("user32.dll")]
    private static extern uint GetWindowThreadProcessId(IntPtr windowHandle, out uint lpdwProcessId);

    
    [DllImport("user32.dll", CharSet = CharSet.Auto)]
    static extern IntPtr SendMessage(IntPtr windowHandle, uint message, IntPtr wParam, [Out] StringBuilder lParam);
    
    public static string GetWindowTextRaw(IntPtr windowHandle)
    {
        // ReSharper disable twice InconsistentNaming
        const uint WM_GETTEXTLENGTH = 0x000E;
        const uint WM_GETTEXT = 0x000D;
        // Allocate correct string length first
        int length = (int)SendMessage(windowHandle, WM_GETTEXTLENGTH, IntPtr.Zero, null);
        StringBuilder sb = new (length + 1);
        SendMessage(windowHandle, WM_GETTEXT, sb.Capacity, sb);
        return sb.ToString();
    }

    public void Dispose()
    {
        KeepUpdating = false;
    }
}

